#!/usr/bin/python
# Onkyo control
"""
	File        : onkyocmd.py
        Description : Controll onkyo receiver power, input and volume
	Version     : 0.1.0
"""
# Import libraries
import socket
import time
import threading
import Queue
import argparse

#Initialize

#DEFINES
__MESSAGE_HEADER_1 = 'ISCP\x00\x00\x00\x10\x00\x00\x00'
__MESSAGE_HEADER_2 = '\x01\x00\x00\x00'
__MESSAGE_TRAILER = '\x0D\x0A'
__MESSAGE_POWER = '!1PWR'
__MESSAGE_POWER2 = '!1ZPW'
__MESSAGE_VOLUME = '!1MVL'
__MESSAGE_VOLUME2 = '!1ZVL'
__MESSAGE_INPUT = '!1SLI'
__MESSAGE_INPUT2 = '!1SLZ'
__MESSAGE_DISCOVER = '!xECNQSTN'

UDP_PORT = 60128
BUFFER_SIZE = 4096

strCurrentPowerState = ''
strCurrentInput = ''
strCurrentVolume = ''

def createISCPFrame(message):
  ISCPFrame = __MESSAGE_HEADER_1+chr(len(message)+1)+__MESSAGE_HEADER_2+message+__MESSAGE_TRAILER
  return ISCPFrame

def interpretData(strData):
  global strCurrentVolume
  global strCurrentPowerState
  global strCurrentInput

  if strData.find('MVL', 0, len(strData)) != -1:
    strCurrentVolume = str(int(strData[strData.find('MVL', 0, len(strData))+3:strData.find('MVL', 0, len(strData))+5],16))

  if strData.find('ZVL', 0, len(strData)) != -1:
    if (strData[strData.find('ZVL', 0, len(strData))+3:strData.find('ZVL', 0, len(strData))+5]=='N/'):
      strCurrentVolume='N/A'
    else:
      strCurrentVolume = str(int(strData[strData.find('ZVL', 0, len(strData))+3:strData.find('ZVL', 0, len(strData))+5],16))

  if strData.find('PWR', 0, len(strData)) != -1:
    if (strData[strData.find('PWR', 0, len(strData))+3:strData.find('PWR', 0, len(strData))+5]=='00'):
      strCurrentPowerState = 'Off'
    else:
      strCurrentPowerState = 'On'

  if strData.find('ZPW', 0, len(strData)) != -1:
    if (strData[strData.find('ZPW', 0, len(strData))+3:strData.find('ZPW', 0, len(strData))+5]=='00'):
      strCurrentPowerState = 'Off'
    else:
      strCurrentPowerState = 'On'

  if strData.find('SLI', 0, len(strData)) != -1:
    strCurrentInput = str(int(strData[strData.find('SLI', 0, len(strData))+3:strData.find('SLI', 0, len(strData))+5],16))

  if strData.find('SLZ', 0, len(strData)) != -1:
    strCurrentInput = str(int(strData[strData.find('SLZ', 0, len(strData))+3:strData.find('SLZ', 0, len(strData))+5],16))

class receiveeISCPDataThread (threading.Thread):
    def __init__(self,queeISCPData):
        threading.Thread.__init__(self)
        self._stop = threading.Event()
    def run(self):
          data = ''
          while self._stop.isSet()==False:
            while len(data) == 0 and self._stop.isSet()==False:
              try:
                data, addr = sockTCP.recvfrom(BUFFER_SIZE, socket.MSG_DONTWAIT)
              except:
                pass
              time.sleep(0.01)
            if data.find('ISCP', 0, len(data)) != -1:
              intStartOfNextSegment = data.find('!', 0, len(data))
              queeISCPData.put(data[intStartOfNextSegment:len(data)])
            data = ''
          exit()
    def stop(self):
          self._stop.set()

class receiveDiscoveryDataThread (threading.Thread):
    def __init__(self,queDiscoveryData):
        threading.Thread.__init__(self)
    def run(self):
          data = ''
          intDiscoveryDataReceived = 0
          intLoopCounter = 0
          while intDiscoveryDataReceived==0 and intLoopCounter!=10:
            intLoopCounter += 1
            try:
              data, addr = sockUDP.recvfrom(BUFFER_SIZE, socket.MSG_DONTWAIT)
            except:
              pass
            time.sleep(0.1)
            if data.find('ECN', 0, len(data)) != -1:
              intStartOfNextSegment = data.find('!', 0, len(data))
              strType = data[intStartOfNextSegment+1:intStartOfNextSegment+2]
              if strType == '1':
                strModel = data[intStartOfNextSegment+5:data.find('/', intStartOfNextSegment+5, len(data))]
                intStartOfNextSegment=data.find('/', intStartOfNextSegment, len(data))+1
                strPortNumber = data[intStartOfNextSegment:data.find('/', intStartOfNextSegment, len(data))]
                intStartOfNextSegment=data.find('/', intStartOfNextSegment, len(data))+1
                strRegion = data[intStartOfNextSegment:data.find('/', intStartOfNextSegment, len(data))]
                intStartOfNextSegment=data.find('/', intStartOfNextSegment, len(data))+1
                strMAC = data[intStartOfNextSegment:intStartOfNextSegment+12]
                queDiscoveryData.put('AV Receiver or Stereo Receiver')
                queDiscoveryData.put(strModel)
                if strRegion == 'DX':
                  queDiscoveryData.put('North American model')
                if strRegion == 'JJ':
                  queDiscoveryData.put('Japanese model')
                if strRegion == 'XX':
                  queDiscoveryData.put('European or Asian model')
                queDiscoveryData.put(addr[0])
                queDiscoveryData.put(strPortNumber)
                queDiscoveryData.put(strMAC)
                intDiscoveryDataReceived=1
            data=''

          exit()



# Main code here

parser = argparse.ArgumentParser(description='Controll Onkyo Receiver.')
parser.add_argument("--power",choices=['on','off'], help="Power on the receiver.")
parser.add_argument("--volume",choices=xrange(0,73), help="Set volume", type=int)
parser.add_argument("--input",choices=xrange(0,47), help="Set input", type=int)
parser.add_argument("--zone2", help="Perform the actions on zone 2.", action="store_true")
parser.add_argument("--state", help="Display the current Power, Input and Volume.", action="store_true")
args = parser.parse_args()
if args.power==None and args.volume==None and args.input==None and args.state==None:
  print "Nothing to do."
else:
  print('Starting autodiscover')
  queDiscoveryData = Queue.Queue()
  sockUDP = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
  sockUDP.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
  sockUDP.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
  intDiscoverySuccess = 0
  intUDPPacketSend = 0
  intLoopCounter2 = 0
  while intDiscoverySuccess == 0 and intLoopCounter2 < 3:
    intLoopCounter2 +=1
    sockUDP = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sockUDP.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    sockUDP.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sockUDP.bind(('0.0.0.0', 60128))
    try:
      sockUDP.sendto(createISCPFrame(__MESSAGE_DISCOVER), ('<broadcast>', 60128))
      intUDPPacketSend = 1
    except:
      intUDPPacketSend = 0
      print('Network not yet ready')
    if intUDPPacketSend == 1:
      threadDiscovery = receiveDiscoveryDataThread(queDiscoveryData)
      threadDiscovery.start()
      intLoopCounter = 0
      while threading.activeCount() > 1 and intLoopCounter < 20:
        time.sleep(0.1)
        intLoopCounter+=1
        if queDiscoveryData.qsize()==6:
          strType = queDiscoveryData.get()
          strModel = queDiscoveryData.get()
          strRegion = queDiscoveryData.get()
          strIPAdress = queDiscoveryData.get()
          inteISCPPort = int(queDiscoveryData.get())
          strMAC = queDiscoveryData.get()
          print('Type:       '+strType)
          print('Model:      '+strModel)
          print('Region:     '+strRegion)
          print('IP adress:  '+strIPAdress)
          print('eISCP port: '+str(inteISCPPort))
          print('MAC:        '+strMAC)
          intDiscoverySuccess = 1
    sockUDP.close()

  if intDiscoverySuccess == 1:
    print('Connecting to: '+strIPAdress+' on port: '+str(inteISCPPort))

    sockTCP = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    intConnected = 0

    try:
      sockTCP.connect((strIPAdress, inteISCPPort))
      intConnected = 1
    except:
      print('Cannot connect to receiver')
      intConnected = 0

    if intConnected == 1:
      queeISCPData = Queue.Queue()
      threadeISCPData = receiveeISCPDataThread(queeISCPData)
      threadeISCPData.start()

      if args.state==True:
        print('Requesting current states.')
        if args.zone2==False:
          sockTCP.send(createISCPFrame(__MESSAGE_POWER+'QSTN'))
        else:
          sockTCP.send(createISCPFrame(__MESSAGE_POWER2+'QSTN'))
        time.sleep(1)
        if args.zone2==False:
          sockTCP.send(createISCPFrame(__MESSAGE_INPUT+'QSTN'))
        else:
          sockTCP.send(createISCPFrame(__MESSAGE_INPUT2+'QSTN'))
        time.sleep(1)
        if args.zone2==False:
          sockTCP.send(createISCPFrame(__MESSAGE_VOLUME+'QSTN'))
        else:
          sockTCP.send(createISCPFrame(__MESSAGE_VOLUME2+'QSTN'))
        time.sleep(1)

        while queeISCPData.qsize() > 0:
          interpretData(queeISCPData.get())

        print('Current Power : '+strCurrentPowerState)
        print('Current Input : '+strCurrentInput)
        print('Current Volume: '+strCurrentVolume) 
      
      if args.power!=None:
        if args.power == "off":
          print('Turning off Receiver')
          if args.zone2==False:
            sockTCP.send(createISCPFrame(__MESSAGE_POWER+'00'))
          else:
            sockTCP.send(createISCPFrame(__MESSAGE_POWER2+'00'))
        else:
          print('Turning on Receiver')
          if args.zone2==False:
            sockTCP.send(createISCPFrame(__MESSAGE_POWER+'01'))
          else:
            sockTCP.send(createISCPFrame(__MESSAGE_POWER2+'01'))
        time.sleep(5)
      if args.input!=None:
        print('Selecting Input')
        strInput = str(hex(args.input))[2:len(hex(args.input))]
        if len(strInput)==1:
          strInput = '0'+strInput
        if args.zone2==False:
          sockTCP.send(createISCPFrame(__MESSAGE_INPUT+strInput))
        else:
          sockTCP.send(createISCPFrame(__MESSAGE_INPUT2+strInput))
        time.sleep(1)
      if args.volume!=None:
        print('Setting Volume')
        strVolume = str(hex(args.volume))
        if len(strVolume) == 1:
          strVolume = '0'+strDesiredVolume
        if args.zone2==False:
          sockTCP.send(createISCPFrame(__MESSAGE_VOLUME+strVolume))
        else:
          sockTCP.send(createISCPFrame(__MESSAGE_VOLUME+strVolume))
        time.sleep(1)

    threadeISCPData.stop()
    sockTCP.close()
